/*-----------------------------------------------------------------------*/
/*  (C) Copyright Christian Monstein HB9SCT 8807 Freienbach              */
/*-----------------------------------------------------------------------*/
/*  Programmname: Sternzei.CPP                                           */
/*                                                                       */
/*  Ausgabe: V2.1     Datum: 03.06.2011    Autor: Chr. Monstein          */
/*                                                                       */
/*  Zweck: Berechnet nach MEEUS die heliozentrischen Koo der Erde und LHA*/
/*                                                                       */
/*  Source: Sternzei.CPP    Compiler: C++5.01                            */
/*                                                                       */
/*-----------------------------------------------------------------------*/

// Created by: Jeffrey Sax   01.11.1992   MEEUS...
// Updated by: Chr. Monstein 06.04.1997   reduziert fr MUON-Telescope
// Updated by: Chr. Monstein 01.05.1998   converted to C++
// Updated by: Chr. Monstein 01.04.1999   JD included
// Updated by: Chr. Monstein 29.11.2008   Sunrise, Sunset included
// Updated by: Chr. Monstein 09.12.2008   configureable
// Updated by: A. James      09.04.2009   added 'delay' time to cfg
//                                        write file only once per day 
// Updated by: Chr. Monstein 03.06.2011   fr alle Lngengrade mit Sort()

//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include <stdio.h>
#include <math.h>
#include <fstream.h>
#include <printers.hpp>

#include "aa.h"
#include "helio.h"
#include "Sternzei.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"

TForm1 *Form1;
int timezone = 0; // PC must be set to UT
int planet   = 0; // 0=sun, ...3=earth
int FCx      = 63; // focuscode
int mmode    = 3; // measuring mode
int monitoring = 0; // from cfg

double heigth;
double laenge;
double breite;
double delaytime;
double rightascension, declination;
double JulianDay;

char cfgpath[240];
char start_time[20];
char restart_time[20];
char stop_time[20];

char TRadio_Mon_Start[20];

//---------------------------------------------------------------------------
void __fastcall TForm1::ReadLocalPar(void)
{
  char buff[240];
  char LonCode, LatCode;
  AnsiString st;
  double horizon;

  ifstream infile("autosched.cfg");
  while (infile)
  {
      infile.getline(buff,240);
      st = LowerCase(AnsiString(buff));
      if (buff[0]=='[')
      {
        if (strstr(st.c_str(),"[longitude]=")) sscanf(buff,"[longitude]=%lf",&laenge);
        if (strstr(st.c_str(),"[latitude]="))  sscanf(buff,"[latitude]=%lf" ,&breite);
        if (strstr(st.c_str(),"[height]="))    sscanf(buff,"[height]=%lf"   ,&heigth);
        if (strstr(st.c_str(),"[horizon]="))     sscanf(buff,"[horizon]=%lf"   ,&horizon);
        if (strstr(st.c_str(),"[focuscode]=")) sscanf(buff,"[focuscode]=%u" ,&FCx);
        if (strstr(st.c_str(),"[measuremode]=")) sscanf(buff,"[measuremode]=%u" ,&mmode);
        if (strstr(st.c_str(),"[cfgpath]="))   sscanf(buff,"[cfgpath]=%s"   ,&cfgpath);
        if (strstr(st.c_str(),"[monitoring]="))   sscanf(buff,"[monitoring]=%i"   ,&monitoring);
      }
  }
  infile.close();
  delaytime = horizon/15.0; // conversion degree -> hours

  if (monitoring==1)
  {
     Form1->Height=136;
     Bevel3->Height=70;
     Label4->Visible=true;
  }
  else
  {
     Form1->Height=120;
     Bevel3->Height=54;
     Label4->Visible=false;
  }
}
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
    : TForm(Owner)
{
    Timer1->Interval=999;
    Timer1->Enabled = true;

    ReadLocalPar(); // read cfg-file
    Timer1Timer(0);
    Button1Click(0);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::Timer1Timer(TObject *Sender)
{
    double lha=0, azi=0, ele=0;
    double s_rise, s_trans, s_set;
    char st[80];
    Word Year, Month, Day, Hour, Min, Sec, MSec;
    double h_dec, temp;
    static int old_min=99;
    int act_min;
    static bool first = true;

    TDateTime dtPresent = Now();
    DecodeDate(dtPresent, Year, Month, Day);
    sprintf(st,"Date: %02u-%02u-%04u",Day,Month,Year);
    Label1->Caption = st;

    DecodeTime(dtPresent, Hour, Min, Sec, MSec);
    act_min = Min;
    sprintf(st,"Local time: %02u:%02u:%02u",Hour,Min,Sec);
    Label2->Caption = st;

    h_dec=(double)Hour+(double)Min/60.0+(double)Sec/3600.0;
    AstroCalc(laenge, breite, timezone, heigth, planet,
      Year, Month, Day, h_dec,
      &lha, &azi, &ele, &rightascension, &declination,
      &JulianDay, &s_rise, &s_trans, &s_set);

    float t1 = s_rise/pi2*24.0, t3 = s_trans/pi2*24.0, t2 = s_set/pi2*24.0;
    int h1, m1, h2, m2, h3, m3;
    float tmin, tmon;
    tmin = (t1+delaytime)*60.0/15.0 - 0.0; // 1/4h genau
    h1 = int(tmin/4);
    m1 = 15*int(tmin - 4*h1);
    Hour = int(t1);
    Min  = int(60*(t1-Hour));
    Sec  = int(3600*t1-(Min*60)-Hour*3600);
    sprintf(start_time,"%02i:%02i:%02i",(h1%24),m1,0);
    sprintf(st,"Sunrise=%02i:%02i:%02i -> scheduler.start %s",Hour,Min,Sec,start_time);
    Label9->Caption = st;

    tmin = (t2-delaytime)*60.0/15.0 + 0.5; // 1/4h genau
    h2 = int(tmin/4);
    m2 = 15*int(tmin - 4*h2);
    Hour = int(t2);
    Min  = int(60*(t2-Hour));
    Sec  = int(3600*t2-(Min*60)-Hour*3600);
    sprintf(stop_time,"%02i:%02i:%02i",(h2%24),m2,0);
    sprintf(st,"Sunset=%02i:%02i:%02i -> scheduler.stop %s",Hour,Min,Sec,stop_time);
    Label10->Caption = st;

    tmin = t3*60.0/15.0 + 0.0; // 1/4h genau
    h3 = int(tmin/4);
    m3 = 15*int(tmin - 4*h3);
    Hour = int(t3);
    Min  = int(60*(t3-Hour));
    Sec  = int(3600*t3-(Min*60)-Hour*3600);
    sprintf(restart_time,"%02i:%02i:%02i",(h3%24),m3,0);
    sprintf(st,"Transit=%02i:%02i:%02i -> scheduler.restart %s",Hour,Min,Sec,restart_time);
    Label3->Caption = st;

    //Radio Monitoring
    t1 = fmod(t2+0.5,24);
    tmon = int(t1*60/15.0 + 0.0);
    h1 = int(tmon/4);
    m1 = int(15*(tmon-4*h1));
    Hour = int(t1);
    Min  = int(60*(t1-Hour));
    Sec  = int(3600*t1-(Min*60)-Hour*3600);
    sprintf(TRadio_Mon_Start,"%02i:%02i:%02i",h1,m1,0);
    sprintf(st,"Monitor=%02i:%02i:%02i -> scheduler.ovs %s",Hour,Min,Sec,TRadio_Mon_Start);
    Label4->Caption = st;


    //Neues scheduler file nur 1mal am Tag erstellen
    dtPresent = Now();
    DecodeTime(dtPresent, Hour, Min, Sec, MSec);
    float decnow = Hour*60+Min+Sec/60;

    if ((decnow+30 >= (h1*60+m1)) && first)
    {
        Button1Click(0);
        first = false;
    }
    if ((decnow > (h2*60+m2)) && first == false)
    {
        first = true;
    }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::Button1Click(TObject *Sender)
{
    TStringList *pList = new TStringList();
    char tmp[80];

    sprintf(tmp,"%s,%02i,%01i   // nominal start",start_time,FCx,mmode);
    pList->Add(tmp);
    sprintf(tmp,"%s,%02i,%01i   // restart just for security reason",restart_time,FCx,mmode);
    pList->Add(tmp);
    sprintf(tmp,"%s,%02i,0   // nominal stop",stop_time,FCx);
    pList->Add(tmp);
    if (monitoring==1)
    {
       sprintf(tmp,"%s,%02i,8   // radio monitoring (spectral overview)",TRadio_Mon_Start,FCx);
       pList->Add(tmp);
    }

    //pList->Add("06:00:00,59,3 // restart");
    //pList->Add("12:00:00,59,0 // stop");
    //pList->Add("13:00:00,59,8 // radio-monitoring");
    pList->Sorted = true;
    //ListBox1->Items->AddStrings(pList);
    /*
    Label1->Caption = pList->Strings[0];
    Label2->Caption = pList->Strings[1];
    Label3->Caption = pList->Strings[2];
    Label4->Caption = pList->Strings[3];
    */


    ofstream outfile(cfgpath);
    if (outfile)
    {
        outfile << "/*-----------------------------------------------------------------------*/" << endl;
        outfile << "/*  (C) Copyright Institute of Astronomy ETHZ 8092 Zuerich Switzerland   */" << endl;
        outfile << "/*-----------------------------------------------------------------------*/" << endl;
        outfile << "/*                                                                       */" << endl;
        outfile << "/*  File: scheduler.cfg     Revision: 3, 03.06.2011  Chr. Monstein       */" << endl;
        outfile << "/*                                                                       */" << endl;
        outfile << "/*-----------------------------------------------------------------------*/" << endl;
        outfile << " " << endl;
        outfile << "/* Automatically created by: SchedulerGeni.exe on " << DateTimeToStr(Now()).c_str() << endl;
        outfile << " " << endl;
        outfile << "/* Each schedule-entry is composed of:" << endl;
        outfile << "/* - Starttime hh:mm:ss (UT) + delimiter" << endl;
        outfile << "/* - focuscodes decimal (00...63) to be applied + delimiter" << endl;
        outfile << "/* - Planned measurement-mode (0, 2, 3, 8)" << endl;
        outfile << "/* - additional comment initiated by //" << endl;
        outfile << "/* each entry will be repeated automatically every day" << endl;
        outfile << "/* empty lines are allowed to separate diffent tasks" << endl;
        outfile << " " << endl;
        outfile << " " << endl;
        //outfile << start_time   << "," << FCx << ",3  // start" << endl;
        outfile << pList->Strings[0].c_str() << endl;

        //outfile << restart_time << "," << FCx << ",3  // restart" << endl;
        outfile << pList->Strings[1].c_str() << endl;

        //outfile << stop_time    << "," << FCx << ",0  // stop" << endl;
        outfile << pList->Strings[2].c_str() << endl;

        if (monitoring==1)
        {
           //outfile << TRadio_Mon_Start << "," << FCx << ",8  // radio monitoring" << endl;
           outfile << pList->Strings[3].c_str() << endl;
        }
        outfile.close();
    }
    delete pList;
}
//---------------------------------------------------------------------------



